@@ -21,6 +21,7 @@
 #include "action.h"
 
 #include "default_configuration.h"
+#include "root_menu.h"
 #include "server.h"
 #include "conf/decode.h"
 #include "conf/plist.h"
@@ -29,6 +30,7 @@
 
 #define WLR_USE_UNSTABLE
 #include <wlr/backend/session.h>
+#include <wlr/types/wlr_cursor.h>
 #include <wlr/types/wlr_keyboard.h>
 #undef WLR_USE_UNSTABLE
 
@@ -98,6 +100,7 @@ const wlmcfg_enum_desc_t wlmaker_action_desc[] = {
     WLMCFG_ENUM("InhibitLockBegin", WLMAKER_ACTION_LOCK_INHIBIT_BEGIN),
     WLMCFG_ENUM("InhibitLockEnd", WLMAKER_ACTION_LOCK_INHIBIT_END),
     WLMCFG_ENUM("LaunchTerminal", WLMAKER_ACTION_LAUNCH_TERMINAL),
+    WLMCFG_ENUM("ShellExecute", WLMAKER_ACTION_SHELL_EXECUTE),
 
     WLMCFG_ENUM("WorkspacePrevious", WLMAKER_ACTION_WORKSPACE_TO_PREVIOUS),
     WLMCFG_ENUM("WorkspaceNext", WLMAKER_ACTION_WORKSPACE_TO_NEXT),
@@ -107,8 +110,16 @@ const wlmcfg_enum_desc_t wlmaker_action_desc[] = {
 
     WLMCFG_ENUM("WindowRaise", WLMAKER_ACTION_WINDOW_RAISE),
     WLMCFG_ENUM("WindowLower", WLMAKER_ACTION_WINDOW_LOWER),
-    WLMCFG_ENUM("WindowFullscreen", WLMAKER_ACTION_WINDOW_TOGGLE_FULLSCREEN),
-    WLMCFG_ENUM("WindowMaximize", WLMAKER_ACTION_WINDOW_TOGGLE_MAXIMIZED),
+    WLMCFG_ENUM("WindowToggleFullscreen", WLMAKER_ACTION_WINDOW_TOGGLE_FULLSCREEN),
+    WLMCFG_ENUM("WindowToggleMaximized", WLMAKER_ACTION_WINDOW_TOGGLE_MAXIMIZED),
+
+    WLMCFG_ENUM("WindowMaximize", WLMAKER_ACTION_WINDOW_MAXIMIZE),
+    WLMCFG_ENUM("WindowUnmaximize", WLMAKER_ACTION_WINDOW_UNMAXIMIZE),
+    WLMCFG_ENUM("WindowFullscreen", WLMAKER_ACTION_WINDOW_FULLSCREEN),
+    WLMCFG_ENUM("WindowShade", WLMAKER_ACTION_WINDOW_SHADE),
+    WLMCFG_ENUM("WindowUnshade", WLMAKER_ACTION_WINDOW_UNSHADE),
+
+    WLMCFG_ENUM("RootMenu", WLMAKER_ACTION_ROOT_MENU),
 
     WLMCFG_ENUM("SwitchToVT1", WLMAKER_ACTION_SWITCH_TO_VT1),
     WLMCFG_ENUM("SwitchToVT2", WLMAKER_ACTION_SWITCH_TO_VT2),
@@ -168,9 +179,10 @@ void wlmaker_action_unbind_keys(wlmaker_action_handle_t *handle_ptr)
 
 /* ------------------------------------------------------------------------- */
 void wlmaker_action_execute(wlmaker_server_t *server_ptr,
-                            wlmaker_action_t action)
+                            wlmaker_action_t action,
+                            void *arg_ptr)
 {
-    wlmtk_workspace_t *wlmtk_workspace_ptr;
+    wlmtk_workspace_t *workspace_ptr, *next_workspace_ptr;
     wlmtk_window_t *window_ptr;
 
     switch (action) {
@@ -201,6 +213,14 @@ void wlmaker_action_execute(wlmaker_server_t *server_ptr,
         }
         break;
 
+    case WLMAKER_ACTION_SHELL_EXECUTE:
+        if (NULL == arg_ptr) {
+            bs_log(BS_ERROR, "Invalid argument NULL for 'Execute'.");
+        } else if (0 == fork()) {
+            execl("/bin/sh", "/bin/sh", "-c", arg_ptr, (void *)NULL);
+        }
+        break;
+
     case WLMAKER_ACTION_WORKSPACE_TO_PREVIOUS:
         wlmtk_root_switch_to_previous_workspace(server_ptr->root_ptr);
         break;
@@ -232,9 +252,9 @@ void wlmaker_action_execute(wlmaker_server_t *server_ptr,
         break;
 
     case WLMAKER_ACTION_WINDOW_TOGGLE_FULLSCREEN:
-        wlmtk_workspace_ptr = wlmtk_root_get_current_workspace(
+        workspace_ptr = wlmtk_root_get_current_workspace(
             server_ptr->root_ptr);
-        window_ptr = wlmtk_workspace_get_activated_window(wlmtk_workspace_ptr);
+        window_ptr = wlmtk_workspace_get_activated_window(workspace_ptr);
         if (NULL != window_ptr) {
             wlmtk_window_request_fullscreen(
                 window_ptr, !wlmtk_window_is_fullscreen(window_ptr));
@@ -242,15 +262,119 @@ void wlmaker_action_execute(wlmaker_server_t *server_ptr,
         break;
 
     case WLMAKER_ACTION_WINDOW_TOGGLE_MAXIMIZED:
-        wlmtk_workspace_ptr = wlmtk_root_get_current_workspace(
+        workspace_ptr = wlmtk_root_get_current_workspace(
             server_ptr->root_ptr);
-        window_ptr = wlmtk_workspace_get_activated_window(wlmtk_workspace_ptr);
+        window_ptr = wlmtk_workspace_get_activated_window(workspace_ptr);
         if (NULL != window_ptr) {
             wlmtk_window_request_maximized(
                 window_ptr, !wlmtk_window_is_maximized(window_ptr));
         }
         break;
 
+    case WLMAKER_ACTION_WINDOW_MAXIMIZE:
+        workspace_ptr = wlmtk_root_get_current_workspace(
+            server_ptr->root_ptr);
+        window_ptr = wlmtk_workspace_get_activated_window(workspace_ptr);
+        if (NULL != window_ptr) {
+            wlmtk_window_request_maximized(window_ptr, true);
+        }
+        break;
+
+    case WLMAKER_ACTION_WINDOW_UNMAXIMIZE:
+        workspace_ptr = wlmtk_root_get_current_workspace(
+            server_ptr->root_ptr);
+        window_ptr = wlmtk_workspace_get_activated_window(workspace_ptr);
+        if (NULL != window_ptr) {
+            wlmtk_window_request_maximized(window_ptr, false);
+        }
+        break;
+
+    case WLMAKER_ACTION_WINDOW_FULLSCREEN:
+        workspace_ptr = wlmtk_root_get_current_workspace(
+            server_ptr->root_ptr);
+        window_ptr = wlmtk_workspace_get_activated_window(workspace_ptr);
+        if (NULL != window_ptr) {
+            wlmtk_window_request_fullscreen(window_ptr, true);
+        }
+        break;
+
+    case WLMAKER_ACTION_WINDOW_SHADE:
+        workspace_ptr = wlmtk_root_get_current_workspace(
+            server_ptr->root_ptr);
+        window_ptr = wlmtk_workspace_get_activated_window(workspace_ptr);
+        if (NULL != window_ptr) {
+            wlmtk_window_request_shaded(window_ptr, true);
+        }
+        break;
+
+    case WLMAKER_ACTION_WINDOW_UNSHADE:
+        workspace_ptr = wlmtk_root_get_current_workspace(
+            server_ptr->root_ptr);
+        window_ptr = wlmtk_workspace_get_activated_window(workspace_ptr);
+        if (NULL != window_ptr) {
+            wlmtk_window_request_shaded(window_ptr, false);
+        }
+        break;
+
+    case WLMAKER_ACTION_WINDOW_TO_NEXT_WORKSPACE:
+        workspace_ptr = wlmtk_root_get_current_workspace(
+            server_ptr->root_ptr);
+        window_ptr = wlmtk_workspace_get_activated_window(workspace_ptr);
+        next_workspace_ptr = wlmtk_workspace_from_dlnode(
+            wlmtk_dlnode_from_workspace(workspace_ptr)->next_ptr);
+        if (NULL != window_ptr &&
+            NULL != next_workspace_ptr) {
+            wlmtk_workspace_unmap_window(workspace_ptr, window_ptr);
+            wlmtk_workspace_map_window(next_workspace_ptr, window_ptr);
+        }
+        break;
+
+    case WLMAKER_ACTION_WINDOW_TO_PREVIOUS_WORKSPACE:
+        workspace_ptr = wlmtk_root_get_current_workspace(
+            server_ptr->root_ptr);
+        window_ptr = wlmtk_workspace_get_activated_window(workspace_ptr);
+        next_workspace_ptr = wlmtk_workspace_from_dlnode(
+            wlmtk_dlnode_from_workspace(workspace_ptr)->prev_ptr);
+        if (NULL != window_ptr &&
+            NULL != next_workspace_ptr) {
+            wlmtk_workspace_unmap_window(workspace_ptr, window_ptr);
+            wlmtk_workspace_map_window(next_workspace_ptr, window_ptr);
+        }
+        break;
+
+    case WLMAKER_ACTION_WINDOW_CLOSE:
+        workspace_ptr = wlmtk_root_get_current_workspace(
+            server_ptr->root_ptr);
+        window_ptr = wlmtk_workspace_get_activated_window(workspace_ptr);
+        if (NULL != window_ptr) {
+            wlmtk_window_request_close(window_ptr);
+        }
+        break;
+
+    case WLMAKER_ACTION_ROOT_MENU:
+        // TODO(kaeser@gubbe.ch): Clean up.
+        if (NULL != server_ptr->root_menu_ptr &&
+            NULL == wlmtk_window_get_workspace(
+                wlmaker_root_menu_window(server_ptr->root_menu_ptr))) {
+            wlmtk_workspace_map_window(
+                wlmtk_root_get_current_workspace(server_ptr->root_ptr),
+                wlmaker_root_menu_window(server_ptr->root_menu_ptr));
+            wlmtk_window_set_position(
+                wlmaker_root_menu_window(server_ptr->root_menu_ptr),
+                server_ptr->cursor_ptr->wlr_cursor_ptr->x,
+                server_ptr->cursor_ptr->wlr_cursor_ptr->y);
+            wlmtk_workspace_confine_within(
+                wlmtk_root_get_current_workspace(server_ptr->root_ptr),
+                wlmaker_root_menu_window(server_ptr->root_menu_ptr));
+            wlmtk_menu_set_mode(
+                wlmaker_root_menu_menu(server_ptr->root_menu_ptr),
+                WLMTK_MENU_MODE_NORMAL);
+            wlmtk_menu_set_open(
+                wlmaker_root_menu_menu(server_ptr->root_menu_ptr),
+                true);
+        }
+        break;
+
     case WLMAKER_ACTION_SWITCH_TO_VT1:
     case WLMAKER_ACTION_SWITCH_TO_VT2:
     case WLMAKER_ACTION_SWITCH_TO_VT3:
@@ -409,7 +533,8 @@ bool _wlmaker_action_bound_callback(const wlmaker_key_combo_t *key_combo_ptr)
 
     wlmaker_action_execute(
         action_binding_ptr->handle_ptr->server_ptr,
-        action_binding_ptr->action);
+        action_binding_ptr->action,
+        NULL);
     return true;
 }
 
