@@ -35,6 +35,7 @@ typedef enum {
     WLMAKER_ACTION_LOCK_INHIBIT_BEGIN,
     WLMAKER_ACTION_LOCK_INHIBIT_END,
     WLMAKER_ACTION_LAUNCH_TERMINAL,
+    WLMAKER_ACTION_SHELL_EXECUTE,
 
     WLMAKER_ACTION_WORKSPACE_TO_PREVIOUS,
     WLMAKER_ACTION_WORKSPACE_TO_NEXT,
@@ -47,6 +48,17 @@ typedef enum {
     WLMAKER_ACTION_WINDOW_TOGGLE_FULLSCREEN,
     WLMAKER_ACTION_WINDOW_TOGGLE_MAXIMIZED,
 
+    WLMAKER_ACTION_WINDOW_MAXIMIZE,
+    WLMAKER_ACTION_WINDOW_UNMAXIMIZE,
+    WLMAKER_ACTION_WINDOW_FULLSCREEN,
+    WLMAKER_ACTION_WINDOW_SHADE,
+    WLMAKER_ACTION_WINDOW_UNSHADE,
+    WLMAKER_ACTION_WINDOW_CLOSE,
+    WLMAKER_ACTION_WINDOW_TO_NEXT_WORKSPACE,
+    WLMAKER_ACTION_WINDOW_TO_PREVIOUS_WORKSPACE,
+
+    WLMAKER_ACTION_ROOT_MENU,
+
     // Note: Keep these numbered consecutively.
     WLMAKER_ACTION_SWITCH_TO_VT1,
     WLMAKER_ACTION_SWITCH_TO_VT2,
@@ -93,10 +105,12 @@ void wlmaker_action_unbind_keys(wlmaker_action_handle_t *handle_ptr);
  *
  * @param server_ptr
  * @param action
+ * @param arg_ptr
  */
 void wlmaker_action_execute(
     wlmaker_server_t *server_ptr,
-    wlmaker_action_t action);
+    wlmaker_action_t action,
+    void *arg_ptr);
 
 /** Unit test cases. */
 extern const bs_test_case_t   wlmaker_action_test_cases[];
