@@ -78,6 +78,10 @@ static void handle_output_layout_change(
     struct wl_listener *listener_ptr,
     void *data_ptr);
 
+static void _wlmaker_server_unclaimed_button_event_handler(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
+
 /* == Data ================================================================= */
 
 const uint32_t wlmaker_modifier_default_mask = (
@@ -242,6 +246,10 @@ wlmaker_server_t *wlmaker_server_create(
         wlmaker_server_destroy(server_ptr);
         return NULL;
     }
+    wlmtk_util_connect_listener_signal(
+        &wlmtk_root_events(server_ptr->root_ptr)->unclaimed_button_event,
+        &server_ptr->unclaimed_button_event_listener,
+        _wlmaker_server_unclaimed_button_event_handler);
 
     // Session lock manager.
     server_ptr->lock_mgr_ptr = wlmaker_lock_mgr_create(server_ptr);
@@ -350,6 +358,11 @@ wlmaker_server_t *wlmaker_server_create(
 /* ------------------------------------------------------------------------- */
 void wlmaker_server_destroy(wlmaker_server_t *server_ptr)
 {
+    if (NULL != server_ptr->root_menu_ptr) {
+        wlmaker_root_menu_destroy(server_ptr->root_menu_ptr);
+        server_ptr->root_menu_ptr = NULL;
+    }
+
     // We don't destroy a few of the handlers, since wlroots will crash if
     // they are destroyed -- and apparently, wlroots cleans them up anyway.
     // These are:
@@ -418,6 +431,8 @@ void wlmaker_server_destroy(wlmaker_server_t *server_ptr)
     }
 
     if (NULL != server_ptr->root_ptr) {
+        wlmtk_util_disconnect_listener(
+            &server_ptr->unclaimed_button_event_listener);
         wlmtk_root_destroy(server_ptr->root_ptr);
         server_ptr->root_ptr = NULL;
     }
@@ -766,6 +781,41 @@ void handle_output_layout_change(
                            &extents);
 }
 
+/* ------------------------------------------------------------------------- */
+/** Handles unclaimed button events: Right 'down' opens root menu. */
+void _wlmaker_server_unclaimed_button_event_handler(
+    struct wl_listener *listener_ptr,
+    void *data_ptr)
+{
+    wlmaker_server_t *server_ptr = BS_CONTAINER_OF(
+        listener_ptr, wlmaker_server_t, unclaimed_button_event_listener);
+    wlmtk_button_event_t *button_event_ptr = data_ptr;
+
+    if (BTN_RIGHT == button_event_ptr->button &&
+        WLMTK_BUTTON_DOWN == button_event_ptr->type &&
+        NULL != server_ptr->root_menu_ptr &&
+        // TODO(kaeser@gubbe.ch): Clean up.
+        !wlmtk_menu_is_open(
+            wlmaker_root_menu_menu(server_ptr->root_menu_ptr))) {
+        wlmtk_workspace_map_window(
+            wlmtk_root_get_current_workspace(server_ptr->root_ptr),
+            wlmaker_root_menu_window(server_ptr->root_menu_ptr));
+        wlmtk_window_set_position(
+            wlmaker_root_menu_window(server_ptr->root_menu_ptr),
+            server_ptr->cursor_ptr->wlr_cursor_ptr->x,
+            server_ptr->cursor_ptr->wlr_cursor_ptr->y);
+        wlmtk_workspace_confine_within(
+            wlmtk_root_get_current_workspace(server_ptr->root_ptr),
+            wlmaker_root_menu_window(server_ptr->root_menu_ptr));
+        wlmtk_menu_set_mode(
+            wlmaker_root_menu_menu(server_ptr->root_menu_ptr),
+            WLMTK_MENU_MODE_RIGHTCLICK);
+        wlmtk_menu_set_open(
+            wlmaker_root_menu_menu(server_ptr->root_menu_ptr),
+            true);
+    }
+}
+
 /* == Unit tests =========================================================== */
 
 static void test_bind(bs_test_t *test_ptr);
