@@ -62,6 +62,7 @@ typedef bool (*wlmaker_keybinding_callback_t)(const wlmaker_key_combo_t *kc);
 #include "keyboard.h"
 #include "layer_shell.h"
 #include "lock_mgr.h"
+#include "root_menu.h"
 #include "subprocess_monitor.h"
 #include "icon_manager.h"
 #include "xdg_decoration.h"
@@ -197,6 +198,13 @@ struct _wlmaker_server_t {
     /** Temporary: Points to the @ref wlmtk_dock_t of the clip. */
     wlmtk_dock_t              *clip_dock_ptr;
 
+    /** Root menu, when active. NULL when not invoked. */
+    wlmaker_root_menu_t       *root_menu_ptr;
+    /** Parsed contents of the root menu definition, from plist. */
+    wlmcfg_array_t            *root_menu_array_ptr;
+    /** Listener for `unclaimed_button_event` signal raised by `wlmtk_root`. */
+    struct wl_listener        unclaimed_button_event_listener;
+
     /** The current configuration style. */
     wlmaker_config_style_t    style;
 };
