@@ -72,6 +72,15 @@ void wlmtk_popup_fini(wlmtk_popup_t *popup_ptr);
 /** Returns the base @ref wlmtk_element_t. */
 wlmtk_element_t *wlmtk_popup_element(wlmtk_popup_t *popup_ptr);
 
+/**
+ * Adds a further popup to `popup_ptr`.
+ *
+ * @param popup_ptr
+ * @param further_popup_ptr
+ */
+void wlmtk_popup_add_popup(wlmtk_popup_t *popup_ptr,
+                           wlmtk_popup_t *further_popup_ptr);
+
 #ifdef __cplusplus
 }  // extern "C"
 #endif  // __cplusplus
