@@ -36,7 +36,7 @@
 #define WLMTK_WINDOW_MAX_PENDING 64
 
 /** Virtual method table for the window. */
-struct  _wlmtk_window_vmt_t {
+typedef struct {
     /** Destructor. */
     void (*destroy)(wlmtk_window_t *window_ptr);
     /** Virtual method for @ref wlmtk_window_request_minimize. */
@@ -46,7 +46,7 @@ struct  _wlmtk_window_vmt_t {
     /** Virtual method for @ref wlmtk_window_request_resize. */
     void (*request_resize)(wlmtk_window_t *window_ptr,
                            uint32_t edges);
-};
+} wlmtk_window_vmt_t;
 
 /** Pending positional updates for @ref wlmtk_window_t::content_ptr. */
 typedef struct {
@@ -76,6 +76,9 @@ struct _wlmtk_window_t {
     /** Virtual method table. */
     wlmtk_window_vmt_t        vmt;
 
+    /** Events for this window. */
+    wlmtk_window_events_t     events;
+
     /** Box: In `super_bordered`, holds surface, title bar and resizebar. */
     wlmtk_box_t               box;
 
@@ -94,6 +97,10 @@ struct _wlmtk_window_t {
     wlmtk_titlebar_t          *titlebar_ptr;
     /** Resizebar. */
     wlmtk_resizebar_t         *resizebar_ptr;
+    /** The window menu. */
+    wlmtk_menu_t              *window_menu_ptr;
+    /** Listener for then the popup menu requests to be closed. */
+    struct wl_listener        menu_request_close_listener;
 
     /** Window title. Set through @ref wlmtk_window_set_title. */
     char                      *title_ptr;
@@ -105,6 +112,9 @@ struct _wlmtk_window_t {
     /** Pre-alloocated updates. */
     wlmtk_pending_update_t    pre_allocated_updates[WLMTK_WINDOW_MAX_PENDING];
 
+    /** This window's properties. */
+    uint32_t                  properties;
+
     /** Organic size of the window, ie. when not maximized. */
     struct wlr_box            organic_size;
     /** Whether the window has been requested as maximized. */
@@ -189,6 +199,10 @@ static void _wlmtk_window_release_update(
     wlmtk_window_t *window_ptr,
     wlmtk_pending_update_t *update_ptr);
 
+static void _wlmtk_window_menu_request_close_handler(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
+
 /* == Data ================================================================= */
 
 /** Virtual method table for the window's element superclass. */
@@ -206,12 +220,19 @@ static const wlmtk_window_vmt_t _wlmtk_window_vmt = {
     .request_resize = _wlmtk_window_request_resize,
 };
 
+/** Default properties. Override by @ref wlmtk_window_set_properties. */
+static const uint32_t _wlmtk_window_default_properties =
+    WLMTK_WINDOW_PROPERTY_RESIZABLE |
+    WLMTK_WINDOW_PROPERTY_ICONIFIABLE |
+    WLMTK_WINDOW_PROPERTY_CLOSABLE;
+
 /* == Exported methods ===================================================== */
 
 /* ------------------------------------------------------------------------- */
 wlmtk_window_t *wlmtk_window_create(
     wlmtk_content_t *content_ptr,
     const wlmtk_window_style_t *style_ptr,
+    const wlmtk_menu_style_t *menu_style_ptr,
     wlmtk_env_t *env_ptr)
 {
     wlmtk_window_t *window_ptr = logged_calloc(1, sizeof(wlmtk_window_t));
@@ -228,6 +249,23 @@ wlmtk_window_t *wlmtk_window_create(
     window_ptr->content_ptr = content_ptr;
     wlmtk_content_set_window(content_ptr, window_ptr);
 
+    // Create the window menu. It is kept hidden until invoked.
+    window_ptr->window_menu_ptr = wlmtk_menu_create(
+        menu_style_ptr,
+        env_ptr);
+    if (NULL == window_ptr->window_menu_ptr) {
+        _wlmtk_window_fini(window_ptr);
+        return false;
+    }
+    // TODO(kaeser@gubbe.ch): Do not access content element directly.
+    wlmtk_container_add_element(
+        &window_ptr->content_ptr->popup_container,
+        wlmtk_menu_element(window_ptr->window_menu_ptr));
+    wlmtk_util_connect_listener_signal(
+        &wlmtk_menu_events(window_ptr->window_menu_ptr)->request_close,
+        &window_ptr->menu_request_close_listener,
+        _wlmtk_window_menu_request_close_handler);
+
     return window_ptr;
 }
 
@@ -238,6 +276,13 @@ void wlmtk_window_destroy(wlmtk_window_t *window_ptr)
     free(window_ptr);
 }
 
+/* ------------------------------------------------------------------------- */
+wlmtk_window_events_t *wlmtk_window_events(
+    wlmtk_window_t *window_ptr)
+{
+    return &window_ptr->events;
+}
+
 /* ------------------------------------------------------------------------- */
 wlmtk_element_t *wlmtk_window_element(wlmtk_window_t *window_ptr)
 {
@@ -271,11 +316,22 @@ void wlmtk_window_set_activated(
     wlmtk_window_t *window_ptr,
     bool activated)
 {
+    if (window_ptr->activated == activated) return;
+
     window_ptr->activated = activated;
     wlmtk_content_set_activated(window_ptr->content_ptr, activated);
     if (NULL != window_ptr->titlebar_ptr) {
         wlmtk_titlebar_set_activated(window_ptr->titlebar_ptr, activated);
     }
+
+    if (!activated) {
+        wlmtk_window_menu_set_enabled(window_ptr, false);
+
+        // TODO(kaeser@gubbe.ch): Should test this behaviour.
+        if (NULL != window_ptr->workspace_ptr) {
+            wlmtk_workspace_activate_window(window_ptr->workspace_ptr, NULL);
+        }
+    }
 }
 
 /* ------------------------------------------------------------------------- */
@@ -290,8 +346,35 @@ void wlmtk_window_set_server_side_decorated(
     bool decorated)
 {
     if (window_ptr->server_side_decorated == decorated) return;
+
+    if (!decorated && wlmtk_window_is_shaded(window_ptr)) {
+        wlmtk_window_request_shaded(window_ptr, false);
+    }
+
     window_ptr->server_side_decorated = decorated;
     _wlmtk_window_apply_decoration(window_ptr);
+
+}
+
+/* ------------------------------------------------------------------------- */
+void wlmtk_window_set_properties(
+    wlmtk_window_t *window_ptr,
+    uint32_t properties)
+{
+    if (window_ptr->properties == properties) return;
+    window_ptr->properties = properties;
+    _wlmtk_window_apply_decoration(window_ptr);
+
+    if (NULL != window_ptr->titlebar_ptr) {
+        uint32_t properties = 0;
+        if (window_ptr->properties & WLMTK_WINDOW_PROPERTY_ICONIFIABLE) {
+            properties |= WLMTK_TITLEBAR_PROPERTY_ICONIFY;
+        }
+        if (window_ptr->properties & WLMTK_WINDOW_PROPERTY_CLOSABLE) {
+            properties |= WLMTK_TITLEBAR_PROPERTY_CLOSE;
+        }
+        wlmtk_titlebar_set_properties(window_ptr->titlebar_ptr, properties);
+    }
 }
 
 /* ------------------------------------------------------------------------- */
@@ -349,9 +432,11 @@ void wlmtk_window_request_maximized(
     wlmtk_window_t *window_ptr,
     bool maximized)
 {
-    if (window_ptr->shaded) return;
+    wlmtk_workspace_t *workspace_ptr = wlmtk_window_get_workspace(window_ptr);
+    // We don't permit maximizing unmapped windows. Don't know the extents.
+    if (NULL == workspace_ptr) maximized = false;
 
-    BS_ASSERT(NULL != wlmtk_window_get_workspace(window_ptr));
+    if (window_ptr->shaded) return;
     if (window_ptr->maximized == maximized) return;
     if (window_ptr->fullscreen) return;
 
@@ -359,8 +444,7 @@ void wlmtk_window_request_maximized(
 
     struct wlr_box box;
     if (maximized) {
-        box = wlmtk_workspace_get_maximize_extents(
-            wlmtk_window_get_workspace(window_ptr));
+        box = wlmtk_workspace_get_maximize_extents(workspace_ptr);
     } else {
         box = window_ptr->organic_size;
     }
@@ -383,6 +467,7 @@ void wlmtk_window_commit_maximized(
     if (window_ptr->maximized == maximized) return;
 
     window_ptr->maximized = maximized;
+    wl_signal_emit(&window_ptr->events.state_changed, window_ptr);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -447,6 +532,7 @@ void wlmtk_window_commit_fullscreen(
 
     wlmtk_workspace_window_to_fullscreen(
         wlmtk_window_get_workspace(window_ptr), window_ptr, fullscreen);
+    wl_signal_emit(&window_ptr->events.state_changed, window_ptr);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -470,6 +556,7 @@ void wlmtk_window_request_shaded(wlmtk_window_t *window_ptr, bool shaded)
     }
 
     window_ptr->shaded = shaded;
+    wl_signal_emit(&window_ptr->events.state_changed, window_ptr);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -478,6 +565,52 @@ bool wlmtk_window_is_shaded(wlmtk_window_t *window_ptr)
     return window_ptr->shaded;
 }
 
+/* ------------------------------------------------------------------------- */
+void wlmtk_window_menu_set_enabled(
+    wlmtk_window_t *window_ptr,
+    bool enabled)
+{
+    if (!window_ptr->activated) enabled = false;
+
+    // For convenience: Get the menu's element. Note: It must have a parent,
+    // since it's contained within the window.
+    wlmtk_element_t *menu_element_ptr = wlmtk_menu_element(
+        window_ptr->window_menu_ptr);
+    BS_ASSERT(NULL != menu_element_ptr->parent_container_ptr);
+
+    wlmtk_menu_set_open(window_ptr->window_menu_ptr, enabled);
+
+    if (enabled) {
+        wlmtk_menu_set_mode(
+            window_ptr->window_menu_ptr,
+            WLMTK_MENU_MODE_RIGHTCLICK);
+        wlmtk_container_raise_element_to_top(
+            menu_element_ptr->parent_container_ptr,
+            menu_element_ptr);
+        wlmtk_container_pointer_grab(
+            menu_element_ptr->parent_container_ptr,
+            menu_element_ptr);
+
+        if (wlmtk_window_element(window_ptr)->pointer_inside) {
+            wlmtk_element_set_position(
+                menu_element_ptr,
+                wlmtk_window_element(window_ptr)->last_pointer_x, 0);
+        } else {
+            wlmtk_element_set_position(menu_element_ptr, 0, 0);
+        }
+    } else {
+        wlmtk_container_pointer_grab_release(
+            menu_element_ptr->parent_container_ptr,
+            menu_element_ptr);
+    }
+}
+
+/* ------------------------------------------------------------------------- */
+wlmtk_menu_t *wlmtk_window_menu(wlmtk_window_t *window_ptr)
+{
+    return window_ptr->window_menu_ptr;
+}
+
 /* ------------------------------------------------------------------------- */
 void wlmtk_window_request_move(wlmtk_window_t *window_ptr)
 {
@@ -589,6 +722,8 @@ void wlmtk_window_set_workspace(
     wlmtk_workspace_t *workspace_ptr)
 {
     window_ptr->workspace_ptr = workspace_ptr;
+
+    wl_signal_emit(&window_ptr->events.state_changed, window_ptr);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -657,10 +792,13 @@ bool _wlmtk_window_init(
     window_ptr->element_ptr = element_ptr;
 
     wlmtk_window_set_title(window_ptr, NULL);
-    _wlmtk_window_apply_decoration(window_ptr);
+    // Also triggers _wlmtk_window_apply_decoration(window_ptr).
+    wlmtk_window_set_properties(window_ptr, _wlmtk_window_default_properties);
 
     wlmtk_box_add_element_front(&window_ptr->box, element_ptr);
     wlmtk_element_set_visible(element_ptr, true);
+
+    wl_signal_init(&window_ptr->events.state_changed);
     return true;
 }
 
@@ -672,6 +810,17 @@ bool _wlmtk_window_init(
  */
 void _wlmtk_window_fini(wlmtk_window_t *window_ptr)
 {
+    if (NULL != window_ptr->window_menu_ptr) {
+        wlmtk_util_disconnect_listener(
+            &window_ptr->menu_request_close_listener);
+
+        wlmtk_container_remove_element(
+            &window_ptr->content_ptr->popup_container,
+            wlmtk_menu_element(window_ptr->window_menu_ptr));
+        wlmtk_menu_destroy(window_ptr->window_menu_ptr);
+        window_ptr->window_menu_ptr = NULL;
+    }
+
     wlmtk_window_set_server_side_decorated(window_ptr, false);
 
     if (NULL != window_ptr->content_ptr) {
@@ -729,7 +878,25 @@ bool _wlmtk_window_element_pointer_button(
     const wlmtk_button_event_t *button_event_ptr)
 {
     wlmtk_window_t *window_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmtk_window_t, super_bordered.super_container.super_element);
+        element_ptr,
+        wlmtk_window_t,
+        super_bordered.super_container.super_element);
+
+    // In right-click mode: Any out-of-window action will close it.
+    // TODO(kaeser@gubbe.ch): This should be a specific window mode, and should
+    // have a handler method when leaving that mode (eg. left release within
+    // the window).
+    if (window_ptr->properties & WLMTK_WINDOW_PROPERTY_RIGHTCLICK) {
+        bool rv = window_ptr->orig_super_element_vmt.pointer_button(
+            element_ptr, button_event_ptr);
+        if (BTN_RIGHT == button_event_ptr->button &&
+            WLMTK_BUTTON_UP == button_event_ptr->type &&
+            !rv) {
+            wlmtk_window_request_close(window_ptr);
+            return true;
+        }
+        return rv;
+    }
 
     // Permit drag-move with the (hardcoded) modifier.
     // TODO(kaeser@gubbe.ch): This should be changed to make use of "DRAG"
@@ -826,6 +993,14 @@ void _wlmtk_window_create_titlebar(wlmtk_window_t *window_ptr)
         window_ptr->super_bordered.super_container.super_element.env_ptr,
         window_ptr, &window_ptr->style.titlebar);
     BS_ASSERT(NULL != window_ptr->titlebar_ptr);
+    uint32_t properties = 0;
+    if (window_ptr->properties & WLMTK_WINDOW_PROPERTY_ICONIFIABLE) {
+        properties |= WLMTK_TITLEBAR_PROPERTY_ICONIFY;
+    }
+    if (window_ptr->properties & WLMTK_WINDOW_PROPERTY_CLOSABLE) {
+        properties |= WLMTK_TITLEBAR_PROPERTY_CLOSE;
+    }
+    wlmtk_titlebar_set_properties(window_ptr->titlebar_ptr, properties);
     wlmtk_titlebar_set_activated(
         window_ptr->titlebar_ptr, window_ptr->activated);
     wlmtk_element_set_visible(
@@ -877,7 +1052,9 @@ void _wlmtk_window_destroy_titlebar(wlmtk_window_t *window_ptr)
 /** Destroys the resizebar. */
 void _wlmtk_window_destroy_resizebar(wlmtk_window_t *window_ptr)
 {
-    BS_ASSERT(!window_ptr->server_side_decorated || window_ptr->fullscreen);
+    BS_ASSERT(!window_ptr->server_side_decorated ||
+              window_ptr->fullscreen ||
+              !(window_ptr->properties & WLMTK_WINDOW_PROPERTY_RESIZABLE));
 
     if (NULL == window_ptr->resizebar_ptr) return;
 
@@ -896,12 +1073,19 @@ void _wlmtk_window_apply_decoration(wlmtk_window_t *window_ptr)
 
     if (window_ptr->server_side_decorated && !window_ptr->fullscreen) {
         _wlmtk_window_create_titlebar(window_ptr);
-        _wlmtk_window_create_resizebar(window_ptr);
     } else {
         bstyle.width = 0;
         _wlmtk_window_destroy_titlebar(window_ptr);
+    }
+
+    if (window_ptr->server_side_decorated &&
+        !window_ptr->fullscreen &&
+        (window_ptr->properties & WLMTK_WINDOW_PROPERTY_RESIZABLE)) {
+        _wlmtk_window_create_resizebar(window_ptr);
+    } else {
         _wlmtk_window_destroy_resizebar(window_ptr);
     }
+
     wlmtk_bordered_set_style(&window_ptr->super_bordered, &bstyle);
 }
 
@@ -1014,6 +1198,17 @@ void _wlmtk_window_release_update(
     bs_dllist_push_front(&window_ptr->available_updates, &update_ptr->dlnode);
 }
 
+/* ------------------------------------------------------------------------- */
+/** Handles @ref wlmtk_menu_events_t::request_close signals. */
+void _wlmtk_window_menu_request_close_handler(
+    struct wl_listener *listener_ptr,
+    __UNUSED__ void *data_ptr)
+{
+    wlmtk_window_t *window_ptr = BS_CONTAINER_OF(
+        listener_ptr, wlmtk_window_t, menu_request_close_listener);
+    wlmtk_window_menu_set_enabled(window_ptr, false);
+}
+
 /* == Implementation of the fake window ==================================== */
 
 static void _wlmtk_fake_window_request_minimize(wlmtk_window_t *window_ptr);
@@ -1044,8 +1239,9 @@ wlmtk_fake_window_t *wlmtk_fake_window_create(void)
         return NULL;
     }
 
-    fake_window_state_ptr->fake_window.fake_content_ptr = wlmtk_fake_content_create(
-        fake_window_state_ptr->fake_window.fake_surface_ptr);
+    fake_window_state_ptr->fake_window.fake_content_ptr =
+        wlmtk_fake_content_create(
+            fake_window_state_ptr->fake_window.fake_surface_ptr);
     if (NULL == fake_window_state_ptr->fake_window.fake_content_ptr) {
         wlmtk_fake_window_destroy(&fake_window_state_ptr->fake_window);
         return NULL;
@@ -1070,6 +1266,16 @@ wlmtk_fake_window_t *wlmtk_fake_window_create(void)
         &fake_window_state_ptr->fake_window.fake_content_ptr->content,
         fake_window_state_ptr->fake_window.window_ptr);
 
+    wlmtk_menu_style_t ms = {};
+    fake_window_state_ptr->fake_window.window_ptr->window_menu_ptr =
+        wlmtk_menu_create(&ms, NULL);
+
+    wlmtk_container_add_element(
+        &fake_window_state_ptr->fake_window.window_ptr->content_ptr->popup_container,
+        wlmtk_menu_element(fake_window_state_ptr->fake_window.window_ptr->window_menu_ptr));
+    fake_window_state_ptr->fake_window.window_menu_ptr =
+        fake_window_state_ptr->fake_window.window_ptr->window_menu_ptr;
+
     // Extend. We don't save the VMT, since it's for fake only.
     _wlmtk_window_extend(&fake_window_state_ptr->window,
                          &_wlmtk_fake_window_vmt);
@@ -1143,9 +1349,11 @@ static void test_set_title(bs_test_t *test_ptr);
 static void test_request_close(bs_test_t *test_ptr);
 static void test_set_activated(bs_test_t *test_ptr);
 static void test_server_side_decorated(bs_test_t *test_ptr);
+static void test_server_side_decorated_properties(bs_test_t *test_ptr);
 static void test_maximize(bs_test_t *test_ptr);
 static void test_fullscreen(bs_test_t *test_ptr);
 static void test_fullscreen_unmap(bs_test_t *test_ptr);
+static void test_shade(bs_test_t *test_ptr);
 static void test_fake(bs_test_t *test_ptr);
 
 const bs_test_case_t wlmtk_window_test_cases[] = {
@@ -1154,9 +1362,12 @@ const bs_test_case_t wlmtk_window_test_cases[] = {
     { 1, "request_close", test_request_close },
     { 1, "set_activated", test_set_activated },
     { 1, "set_server_side_decorated", test_server_side_decorated },
+    { 1, "set_server_side_decorated_properties",
+      test_server_side_decorated_properties },
     { 1, "maximize", test_maximize },
     { 1, "fullscreen", test_fullscreen },
     { 1, "fullscreen_unmap", test_fullscreen_unmap },
+    { 1, "shade", test_shade },
     { 1, "fake", test_fake },
     { 0, NULL, NULL }
 };
@@ -1166,10 +1377,14 @@ const bs_test_case_t wlmtk_window_test_cases[] = {
 void test_create_destroy(bs_test_t *test_ptr)
 {
     wlmtk_fake_surface_t *fake_surface_ptr = wlmtk_fake_surface_create();
-    wlmtk_window_style_t style = {};
+    wlmtk_window_style_t s = {};
+    wlmtk_menu_style_t ms = {};
     wlmtk_content_t content;
-    wlmtk_content_init(&content, &fake_surface_ptr->surface, NULL);
-    wlmtk_window_t *window_ptr = wlmtk_window_create(&content, &style, NULL);
+    wlmtk_content_init(
+        &content,
+        wlmtk_surface_element(&fake_surface_ptr->surface),
+        NULL);
+    wlmtk_window_t *window_ptr = wlmtk_window_create(&content, &s, &ms, NULL);
     BS_TEST_VERIFY_NEQ(test_ptr, NULL, window_ptr);
     BS_TEST_VERIFY_EQ(test_ptr, window_ptr, content.window_ptr);
 
@@ -1187,7 +1402,7 @@ void test_create_destroy(bs_test_t *test_ptr)
 void test_set_title(bs_test_t *test_ptr)
 {
     wlmtk_fake_window_t *fw_ptr = wlmtk_fake_window_create();
-    BS_ASSERT(NULL != fw_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fw_ptr);
 
     wlmtk_window_set_title(fw_ptr->window_ptr, "Title");
     BS_TEST_VERIFY_STREQ(
@@ -1209,7 +1424,7 @@ void test_set_title(bs_test_t *test_ptr)
 void test_request_close(bs_test_t *test_ptr)
 {
     wlmtk_fake_window_t *fw_ptr = wlmtk_fake_window_create();
-    BS_ASSERT(NULL != fw_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fw_ptr);
 
     wlmtk_window_request_close(fw_ptr->window_ptr);
     BS_TEST_VERIFY_TRUE(
@@ -1224,7 +1439,7 @@ void test_request_close(bs_test_t *test_ptr)
 void test_set_activated(bs_test_t *test_ptr)
 {
     wlmtk_fake_window_t *fw_ptr = wlmtk_fake_window_create();
-    BS_ASSERT(NULL != fw_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fw_ptr);
 
     wlmtk_window_set_activated(fw_ptr->window_ptr, true);
     BS_TEST_VERIFY_TRUE(test_ptr, fw_ptr->fake_content_ptr->activated);
@@ -1239,12 +1454,12 @@ void test_set_activated(bs_test_t *test_ptr)
 /** Tests enabling and disabling server-side decoration. */
 void test_server_side_decorated(bs_test_t *test_ptr)
 {
-    wlmtk_fake_workspace_t *fws_ptr = wlmtk_fake_workspace_create(1024, 768);
-    BS_ASSERT(NULL != fws_ptr);
+    wlmtk_workspace_t *ws_ptr = wlmtk_workspace_create_for_test(1024, 768, 0);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws_ptr);
     wlmtk_fake_window_t *fw_ptr = wlmtk_fake_window_create();
-    BS_ASSERT(NULL != fw_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fw_ptr);
 
-    wlmtk_workspace_map_window(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
+    wlmtk_workspace_map_window(ws_ptr, fw_ptr->window_ptr);
 
     BS_TEST_VERIFY_EQ(test_ptr, NULL, fw_ptr->window_ptr->titlebar_ptr);
     BS_TEST_VERIFY_EQ(test_ptr, NULL, fw_ptr->window_ptr->resizebar_ptr);
@@ -1284,10 +1499,41 @@ void test_server_side_decorated(bs_test_t *test_ptr)
     BS_TEST_VERIFY_EQ(test_ptr, NULL, fw_ptr->window_ptr->titlebar_ptr);
     BS_TEST_VERIFY_EQ(test_ptr, NULL, fw_ptr->window_ptr->resizebar_ptr);
 
-    wlmtk_workspace_unmap_window(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
+    wlmtk_workspace_unmap_window(ws_ptr, fw_ptr->window_ptr);
 
     wlmtk_fake_window_destroy(fw_ptr);
-    wlmtk_fake_workspace_destroy(fws_ptr);
+    wlmtk_workspace_destroy(ws_ptr);
+}
+
+/* ------------------------------------------------------------------------- */
+/** Tests server-side decoration depending on properties. */
+void test_server_side_decorated_properties(bs_test_t *test_ptr)
+{
+    wlmtk_workspace_t *ws_ptr = wlmtk_workspace_create_for_test(1024, 768, 0);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws_ptr);
+    wlmtk_fake_window_t *fw_ptr = wlmtk_fake_window_create();
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fw_ptr);
+
+    wlmtk_workspace_map_window(ws_ptr, fw_ptr->window_ptr);
+    wlmtk_window_set_server_side_decorated(fw_ptr->window_ptr, true);
+    BS_TEST_VERIFY_NEQ(test_ptr, NULL, fw_ptr->window_ptr->titlebar_ptr);
+    BS_TEST_VERIFY_NEQ(test_ptr, NULL, fw_ptr->window_ptr->resizebar_ptr);
+
+    wlmtk_window_set_properties(
+        fw_ptr->window_ptr,
+        WLMTK_WINDOW_PROPERTY_ICONIFIABLE);
+    BS_TEST_VERIFY_NEQ(test_ptr, NULL, fw_ptr->window_ptr->titlebar_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, NULL, fw_ptr->window_ptr->resizebar_ptr);
+
+    wlmtk_window_set_properties(
+        fw_ptr->window_ptr,
+        WLMTK_WINDOW_PROPERTY_RESIZABLE);
+    BS_TEST_VERIFY_NEQ(test_ptr, NULL, fw_ptr->window_ptr->titlebar_ptr);
+    BS_TEST_VERIFY_NEQ(test_ptr, NULL, fw_ptr->window_ptr->resizebar_ptr);
+
+    wlmtk_workspace_unmap_window(ws_ptr, fw_ptr->window_ptr);
+    wlmtk_fake_window_destroy(fw_ptr);
+    wlmtk_workspace_destroy(ws_ptr);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -1295,14 +1541,20 @@ void test_server_side_decorated(bs_test_t *test_ptr)
 void test_maximize(bs_test_t *test_ptr)
 {
     struct wlr_box box;
+    wlmtk_util_test_listener_t l;
 
-    wlmtk_fake_workspace_t *fws_ptr = wlmtk_fake_workspace_create(1024, 768);
-    BS_ASSERT(NULL != fws_ptr);
+    wlmtk_workspace_t *ws_ptr = wlmtk_workspace_create_for_test(1024, 768, 0);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws_ptr);
     wlmtk_fake_window_t *fw_ptr = wlmtk_fake_window_create();
-    BS_ASSERT(NULL != fw_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fw_ptr);
+
+    wlmtk_util_connect_test_listener(
+        &wlmtk_window_events(fw_ptr->window_ptr)->state_changed, &l);
 
     // Window must be mapped to get maximized: Need workspace dimensions.
-    wlmtk_workspace_map_window(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
+    wlmtk_workspace_map_window(ws_ptr, fw_ptr->window_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
+    wlmtk_util_clear_test_listener(&l);
 
     // Set up initial organic size, and verify.
     wlmtk_window_request_position_and_size(fw_ptr->window_ptr, 20, 10, 200, 100);
@@ -1334,6 +1586,7 @@ void test_maximize(bs_test_t *test_ptr)
     wlmtk_window_request_maximized(fw_ptr->window_ptr, true);
     BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_window_is_maximized(fw_ptr->window_ptr));
     wlmtk_fake_window_commit_size(fw_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, 0, l.calls);
     wlmtk_window_commit_maximized(fw_ptr->window_ptr, true);
     box = wlmtk_window_get_position_and_size(fw_ptr->window_ptr);
     BS_TEST_VERIFY_EQ(test_ptr, 0, box.x);
@@ -1341,6 +1594,8 @@ void test_maximize(bs_test_t *test_ptr)
     BS_TEST_VERIFY_EQ(test_ptr, 960, box.width);
     BS_TEST_VERIFY_EQ(test_ptr, 704, box.height);
     BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_window_is_maximized(fw_ptr->window_ptr));
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
+    wlmtk_util_clear_test_listener(&l);
 
     // A second commit: should not overwrite the organic dimension.
     wlmtk_fake_window_commit_size(fw_ptr);
@@ -1349,6 +1604,7 @@ void test_maximize(bs_test_t *test_ptr)
     wlmtk_window_request_maximized(fw_ptr->window_ptr, false);
     BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_window_is_maximized(fw_ptr->window_ptr));
     wlmtk_fake_window_commit_size(fw_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, 0, l.calls);
     wlmtk_window_commit_maximized(fw_ptr->window_ptr, false);
     box = wlmtk_window_get_position_and_size(fw_ptr->window_ptr);
     BS_TEST_VERIFY_EQ(test_ptr, 50, box.x);
@@ -1356,15 +1612,17 @@ void test_maximize(bs_test_t *test_ptr)
     BS_TEST_VERIFY_EQ(test_ptr, 200, box.width);
     BS_TEST_VERIFY_EQ(test_ptr, 100, box.height);
     BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_window_is_maximized(fw_ptr->window_ptr));
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
 
     // TODO(kaeser@gubbe.ch): Define what should happen when a maximized
     // window is moved. Should it lose maximization? Should it not move?
     // Or just move on?
     // Window Maker keeps maximization, but it's ... odd.
 
-    wlmtk_workspace_unmap_window(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
+    wlmtk_workspace_unmap_window(ws_ptr, fw_ptr->window_ptr);
+    wlmtk_util_disconnect_test_listener(&l);
     wlmtk_fake_window_destroy(fw_ptr);
-    wlmtk_fake_workspace_destroy(fws_ptr);
+    wlmtk_workspace_destroy(ws_ptr);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -1372,20 +1630,26 @@ void test_maximize(bs_test_t *test_ptr)
 void test_fullscreen(bs_test_t *test_ptr)
 {
     struct wlr_box box;
+    wlmtk_util_test_listener_t l;
 
-    wlmtk_fake_workspace_t *fws_ptr = wlmtk_fake_workspace_create(1024, 768);
-    BS_ASSERT(NULL != fws_ptr);
+    wlmtk_workspace_t *ws_ptr = wlmtk_workspace_create_for_test(1024, 768, 0);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws_ptr);
     wlmtk_fake_window_t *fw_ptr = wlmtk_fake_window_create();
-    BS_ASSERT(NULL != fw_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fw_ptr);
+
+    wlmtk_util_connect_test_listener(
+        &wlmtk_window_events(fw_ptr->window_ptr)->state_changed, &l);
 
     wlmtk_window_set_server_side_decorated(fw_ptr->window_ptr, true);
-    wlmtk_workspace_map_window(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
+    wlmtk_workspace_map_window(ws_ptr, fw_ptr->window_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
+    wlmtk_util_clear_test_listener(&l);
 
     BS_TEST_VERIFY_TRUE(test_ptr, fw_ptr->fake_content_ptr->activated);
     BS_TEST_VERIFY_EQ(
         test_ptr,
         fw_ptr->window_ptr,
-        wlmtk_workspace_get_activated_window(fws_ptr->workspace_ptr));
+        wlmtk_workspace_get_activated_window(ws_ptr));
 
     // Set up initial organic size, and verify.
     wlmtk_window_request_position_and_size(fw_ptr->window_ptr, 20, 10, 200, 100);
@@ -1403,6 +1667,7 @@ void test_fullscreen(bs_test_t *test_ptr)
     BS_TEST_VERIFY_TRUE(
         test_ptr,
         wlmtk_titlebar_is_activated(fw_ptr->window_ptr->titlebar_ptr));
+    BS_TEST_VERIFY_EQ(test_ptr, 0, l.calls);
 
     // Only after "commit", it will take effect.
     wlmtk_fake_window_commit_size(fw_ptr);
@@ -1413,12 +1678,14 @@ void test_fullscreen(bs_test_t *test_ptr)
     BS_TEST_VERIFY_EQ(test_ptr, 0, box.y);
     BS_TEST_VERIFY_EQ(test_ptr, 1024, box.width);
     BS_TEST_VERIFY_EQ(test_ptr, 768, box.height);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
+    wlmtk_util_clear_test_listener(&l);
 
     BS_TEST_VERIFY_TRUE(test_ptr, fw_ptr->fake_content_ptr->activated);
     BS_TEST_VERIFY_EQ(
         test_ptr,
         fw_ptr->window_ptr,
-        wlmtk_workspace_get_activated_window(fws_ptr->workspace_ptr));
+        wlmtk_workspace_get_activated_window(ws_ptr));
 
     BS_TEST_VERIFY_TRUE(test_ptr, fw_ptr->window_ptr->server_side_decorated);
     BS_TEST_VERIFY_EQ(test_ptr, NULL, fw_ptr->window_ptr->titlebar_ptr);
@@ -1427,6 +1694,7 @@ void test_fullscreen(bs_test_t *test_ptr)
     // Request to end fullscreen. Not taking immediate effect.
     wlmtk_window_request_fullscreen(fw_ptr->window_ptr, false);
     BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_window_is_fullscreen(fw_ptr->window_ptr));
+    BS_TEST_VERIFY_EQ(test_ptr, 0, l.calls);
 
     // Takes effect after commit. We'll want the same position as before.
     wlmtk_fake_window_commit_size(fw_ptr);
@@ -1445,16 +1713,18 @@ void test_fullscreen(bs_test_t *test_ptr)
     BS_TEST_VERIFY_EQ(
         test_ptr,
         fw_ptr->window_ptr,
-        wlmtk_workspace_get_activated_window(fws_ptr->workspace_ptr));
+        wlmtk_workspace_get_activated_window(ws_ptr));
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
 
     BS_TEST_VERIFY_TRUE(test_ptr, fw_ptr->window_ptr->server_side_decorated);
     BS_TEST_VERIFY_NEQ(test_ptr, NULL, fw_ptr->window_ptr->titlebar_ptr);
     BS_TEST_VERIFY_NEQ(test_ptr, NULL, fw_ptr->window_ptr->resizebar_ptr);
 
-    wlmtk_workspace_unmap_window(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
-    wlmtk_fake_window_destroy(fw_ptr);
+    wlmtk_workspace_unmap_window(ws_ptr, fw_ptr->window_ptr);
+    wlmtk_util_disconnect_test_listener(&l);
 
-    wlmtk_fake_workspace_destroy(fws_ptr);
+    wlmtk_fake_window_destroy(fw_ptr);
+    wlmtk_workspace_destroy(ws_ptr);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -1463,18 +1733,17 @@ void test_fullscreen_unmap(bs_test_t *test_ptr)
 {
     struct wlr_box box;
 
-    wlmtk_fake_workspace_t *fws_ptr = wlmtk_fake_workspace_create(1024, 768);
-    BS_ASSERT(NULL != fws_ptr);
-
+    wlmtk_workspace_t *ws_ptr = wlmtk_workspace_create_for_test(1024, 768, 0);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws_ptr);
     wlmtk_fake_window_t *fw_ptr = wlmtk_fake_window_create();
-    BS_ASSERT(NULL != fw_ptr);
-    wlmtk_workspace_map_window(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fw_ptr);
+    wlmtk_workspace_map_window(ws_ptr, fw_ptr->window_ptr);
 
     BS_TEST_VERIFY_TRUE(test_ptr, fw_ptr->fake_content_ptr->activated);
     BS_TEST_VERIFY_EQ(
         test_ptr,
         fw_ptr->window_ptr,
-        wlmtk_workspace_get_activated_window(fws_ptr->workspace_ptr));
+        wlmtk_workspace_get_activated_window(ws_ptr));
 
     // Request fullscreen. Does not take immediate effect.
     wlmtk_window_request_fullscreen(fw_ptr->window_ptr, true);
@@ -1491,16 +1760,67 @@ void test_fullscreen_unmap(bs_test_t *test_ptr)
     BS_TEST_VERIFY_EQ(test_ptr, 768, box.height);
     BS_TEST_VERIFY_TRUE(test_ptr, fw_ptr->fake_content_ptr->activated);
 
-    wlmtk_workspace_unmap_window(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
+    wlmtk_workspace_unmap_window(ws_ptr, fw_ptr->window_ptr);
     BS_TEST_VERIFY_FALSE(test_ptr, fw_ptr->fake_content_ptr->activated);
     BS_TEST_VERIFY_EQ(
         test_ptr,
         NULL,
-        wlmtk_workspace_get_activated_window(fws_ptr->workspace_ptr));
+        wlmtk_workspace_get_activated_window(ws_ptr));
 
     wlmtk_fake_window_destroy(fw_ptr);
 
-    wlmtk_fake_workspace_destroy(fws_ptr);
+    wlmtk_workspace_destroy(ws_ptr);
+}
+
+/* ------------------------------------------------------------------------- */
+/** Verifies that window shading hides the element and raises signal. */
+void test_shade(bs_test_t *test_ptr)
+{
+    wlmtk_util_test_listener_t l;
+
+    wlmtk_fake_window_t *fw_ptr = wlmtk_fake_window_create();
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fw_ptr);
+    wlmtk_util_connect_test_listener(
+        &wlmtk_window_events(fw_ptr->window_ptr)->state_changed, &l);
+
+    BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_window_is_shaded(fw_ptr->window_ptr));
+    BS_TEST_VERIFY_EQ(test_ptr, 0, l.calls);
+
+    // Shading only works on server-side-decorated windows.
+    wlmtk_window_set_server_side_decorated(fw_ptr->window_ptr, true);
+
+    wlmtk_window_request_shaded(fw_ptr->window_ptr, true);
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_window_is_shaded(fw_ptr->window_ptr));
+    BS_TEST_VERIFY_FALSE(
+        test_ptr,
+        wlmtk_content_element(&fw_ptr->fake_content_ptr->content)->visible);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
+    wlmtk_util_clear_test_listener(&l);
+
+    wlmtk_window_request_shaded(fw_ptr->window_ptr, false);
+    BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_window_is_shaded(fw_ptr->window_ptr));
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
+    wlmtk_util_clear_test_listener(&l);
+
+    // Shading not supported on client-side decoration. Must be disabled.
+    wlmtk_window_request_shaded(fw_ptr->window_ptr, true);
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_window_is_shaded(fw_ptr->window_ptr));
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
+    wlmtk_util_clear_test_listener(&l);
+
+    wlmtk_window_set_server_side_decorated(fw_ptr->window_ptr, false);
+    BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_window_is_shaded(fw_ptr->window_ptr));
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
+    wlmtk_util_clear_test_listener(&l);
+
+    // Verify that 'shading' on client decorations does not do anything.
+    wlmtk_window_set_server_side_decorated(fw_ptr->window_ptr, false);
+    wlmtk_window_request_shaded(fw_ptr->window_ptr, true);
+    BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_window_is_shaded(fw_ptr->window_ptr));
+    BS_TEST_VERIFY_EQ(test_ptr, 0, l.calls);
+
+    wlmtk_util_disconnect_test_listener(&l);
+    wlmtk_fake_window_destroy(fw_ptr);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -1508,7 +1828,7 @@ void test_fullscreen_unmap(bs_test_t *test_ptr)
 void test_fake(bs_test_t *test_ptr)
 {
     wlmtk_fake_window_t *fake_window_ptr = wlmtk_fake_window_create();
-    BS_TEST_VERIFY_NEQ(test_ptr, NULL, fake_window_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fake_window_ptr);
     wlmtk_fake_window_destroy(fake_window_ptr);
 }
 
