@@ -401,6 +401,31 @@ struct wlr_box wlmtk_workspace_get_maximize_extents(
     return box;
 }
 
+/* ------------------------------------------------------------------------- */
+void wlmtk_workspace_confine_within(
+    wlmtk_workspace_t *workspace_ptr,
+    wlmtk_window_t *window_ptr)
+{
+    // Only act if the window belongs to this workspace.
+    if (workspace_ptr != wlmtk_window_get_workspace(window_ptr)) return;
+
+    struct wlr_box box = wlmtk_workspace_get_fullscreen_extents(workspace_ptr);
+
+    struct wlr_box elem_box = wlmtk_element_get_dimensions_box(
+        wlmtk_window_element(window_ptr));
+    int x, y;
+    wlmtk_element_get_position(wlmtk_window_element(window_ptr), &x, &y);
+
+
+    int max_x = x - elem_box.x + elem_box.width;
+    if (max_x > box.width) x -= max_x - box.width;
+
+    int max_y = y - elem_box.y + elem_box.height;
+    if (max_y > box.height) y -= max_y - box.height;
+
+    wlmtk_element_set_position(wlmtk_window_element(window_ptr), x, y);
+}
+
 /* ------------------------------------------------------------------------- */
 struct wlr_box wlmtk_workspace_get_fullscreen_extents(
     wlmtk_workspace_t *workspace_ptr)
@@ -615,16 +640,17 @@ void wlmtk_workspace_activate_window(
     if (workspace_ptr->activated_window_ptr == window_ptr) return;
 
     if (NULL != workspace_ptr->activated_window_ptr) {
-        wlmtk_window_set_activated(workspace_ptr->activated_window_ptr, false);
+        wlmtk_window_t *w_ptr = workspace_ptr->activated_window_ptr;
         workspace_ptr->formerly_activated_window_ptr =
             workspace_ptr->activated_window_ptr;
         workspace_ptr->activated_window_ptr = NULL;
+        wlmtk_window_set_activated(w_ptr, false);
     }
 
     if (NULL != window_ptr) {
         if (workspace_ptr->enabled) {
-            wlmtk_window_set_activated(window_ptr, true);
             workspace_ptr->activated_window_ptr = window_ptr;
+            wlmtk_window_set_activated(window_ptr, true);
         }
         workspace_ptr->formerly_activated_window_ptr = window_ptr;
     }
@@ -1008,42 +1034,6 @@ const bs_test_case_t wlmtk_workspace_test_cases[] = {
     { 0, NULL, NULL }
 };
 
-/** Listeners for tests. */
-typedef struct {
-     /** Listener for when the window is mapped. */
-    struct wl_listener        window_mapped_listener;
-    /** Listener for when the window is unmapped. */
-    struct wl_listener        window_unmapped_listener;
-    /** Reports whether the handler was invoked. */
-    bool                      window_mapped_handler_invoked;
-    /** Reports whether the handler was invoked. */
-    bool                      window_unmapped_handler_invoked;
-} _wlmtk_workspace_test_listeners_t;
-
-/** Test handler for @ref wlmtk_root_events_t::window_mapped. */
-static void _wlmtk_workspace_test_handle_window_mapped(
-    struct wl_listener *listener_ptr,
-    __UNUSED__ void *data_ptr)
-{
-    _wlmtk_workspace_test_listeners_t *tl_ptr = BS_CONTAINER_OF(
-        listener_ptr,
-        _wlmtk_workspace_test_listeners_t,
-        window_mapped_listener);
-    tl_ptr->window_mapped_handler_invoked = true;
-}
-
-/** Test handler for @ref wlmtk_root_events_t::window_unmapped. */
-static void _wlmtk_workspace_test_handle_window_unmapped(
-    struct wl_listener *listener_ptr,
-    __UNUSED__ void *data_ptr)
-{
-    _wlmtk_workspace_test_listeners_t *tl_ptr = BS_CONTAINER_OF(
-        listener_ptr,
-        _wlmtk_workspace_test_listeners_t,
-        window_unmapped_listener);
-    tl_ptr->window_unmapped_handler_invoked = true;
-}
-
 /** Tile style used in tests. */
 static const wlmtk_tile_style_t _wlmtk_workspace_test_tile_style = {
     .size = 64
@@ -1102,15 +1092,11 @@ void test_map_unmap(bs_test_t *test_ptr)
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, workspace_ptr);
     wlmtk_root_add_workspace(root_ptr, workspace_ptr);
 
-    _wlmtk_workspace_test_listeners_t test_listeners = {};
-    wlmtk_util_connect_listener_signal(
-        &wlmtk_root_events(root_ptr)->window_mapped,
-        &test_listeners.window_mapped_listener,
-        _wlmtk_workspace_test_handle_window_mapped);
-    wlmtk_util_connect_listener_signal(
-        &wlmtk_root_events(root_ptr)->window_unmapped,
-        &test_listeners.window_unmapped_listener,
-        _wlmtk_workspace_test_handle_window_unmapped);
+    wlmtk_util_test_listener_t mapped, unmapped;
+    wlmtk_util_connect_test_listener(
+        &wlmtk_root_events(root_ptr)->window_mapped, &mapped);
+    wlmtk_util_connect_test_listener(
+        &wlmtk_root_events(root_ptr)->window_unmapped, &unmapped);
 
     bs_dllist_t *wdl_ptr = wlmtk_workspace_get_windows_dllist(workspace_ptr);
     BS_TEST_VERIFY_EQ(test_ptr, 0, bs_dllist_size(wdl_ptr));
@@ -1130,8 +1116,7 @@ void test_map_unmap(bs_test_t *test_ptr)
         fw_ptr->fake_surface_ptr->surface.super_element.wlr_scene_node_ptr);
     BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_window_element(fw_ptr->window_ptr)->visible);
     BS_TEST_VERIFY_EQ(test_ptr, 1, bs_dllist_size(wdl_ptr));
-    BS_TEST_VERIFY_TRUE(
-        test_ptr, test_listeners.window_mapped_handler_invoked);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, mapped.calls);
 
     wlmtk_workspace_unmap_window(workspace_ptr, fw_ptr->window_ptr);
     BS_TEST_VERIFY_EQ(
@@ -1144,11 +1129,10 @@ void test_map_unmap(bs_test_t *test_ptr)
         fw_ptr->fake_surface_ptr->surface.super_element.wlr_scene_node_ptr);
     BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_window_element(fw_ptr->window_ptr)->visible);
     BS_TEST_VERIFY_EQ(test_ptr, 0, bs_dllist_size(wdl_ptr));
-    BS_TEST_VERIFY_TRUE(
-        test_ptr, test_listeners.window_unmapped_handler_invoked);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, unmapped.calls);
 
-    wlmtk_util_disconnect_listener(&test_listeners.window_mapped_listener);
-    wlmtk_util_disconnect_listener(&test_listeners.window_unmapped_listener);
+    wlmtk_util_disconnect_test_listener(&mapped);
+    wlmtk_util_disconnect_test_listener(&unmapped);
     wlmtk_fake_window_destroy(fw_ptr);
     wlmtk_root_remove_workspace(root_ptr, workspace_ptr);
     wlmtk_workspace_destroy(workspace_ptr);
