@@ -40,7 +40,8 @@
 #include "server.h"
 #include "task_list.h"
 
-#include "style_default.h"
+#include "../etc/style.h"
+#include "../etc/root_menu.h"
 
 /** Will hold the value of --config_file. */
 static char *wlmaker_arg_config_file_ptr = NULL;
@@ -48,6 +49,8 @@ static char *wlmaker_arg_config_file_ptr = NULL;
 static char *wlmaker_arg_state_file_ptr = NULL;
 /** Will hold the value of --style_file. */
 static char *wlmaker_arg_style_file_ptr = NULL;
+/** Will hold the value of --root_menu_file. */
+static char *wlmaker_arg_root_menu_file_ptr = NULL;
 
 /** Startup options for the server. */
 static wlmaker_server_options_t wlmaker_server_options = {
@@ -94,6 +97,12 @@ static const bs_arg_t wlmaker_args[] = {
         "will use a built-in default style.",
         NULL,
         &wlmaker_arg_style_file_ptr),
+    BS_ARG_STRING(
+        "root_menu_file",
+        "Optional: Path to a file describing the root menu. If not provided, "
+        "wlmaker will use a built-in definition for the root menu.",
+        NULL,
+        &wlmaker_arg_root_menu_file_ptr),
     BS_ARG_ENUM(
         "log_level",
         "Log level to apply. One of DEBUG, INFO, WARNING, ERROR.",
@@ -276,6 +285,20 @@ bool create_workspaces(
     return rv;
 }
 
+/** Lookup paths for the root menu config file. */
+static const char *_wlmaker_root_menu_fname_ptrs[] = {
+    "~/.wlmaker-root-menu.plist",
+    "/usr/share/wlmaker/root-menu.plist",
+    NULL  // Sentinel.
+};
+
+/** Lookup paths for the style config file. */
+static const char *_wlmaker_style_fname_ptrs[] = {
+    "~/.wlmaker-style.plist",
+    "/usr/share/wlmaker/style.plist",
+    NULL  // Sentinel.
+};
+
 /* == Main program ========================================================= */
 /** The main program. */
 int main(__UNUSED__ int argc, __UNUSED__ const char **argv)
@@ -325,26 +348,41 @@ int main(__UNUSED__ int argc, __UNUSED__ const char **argv)
         config_dict_ptr, &wlmaker_server_options);
     if (NULL == server_ptr) return EXIT_FAILURE;
 
-    // TODO: Should be loaded from file, if given in the config. Or on the
-    // commandline.
-    wlmcfg_dict_t *style_dict_ptr = NULL;
-    if (NULL != wlmaker_arg_style_file_ptr) {
-        style_dict_ptr = wlmcfg_dict_from_object(
-            wlmcfg_create_object_from_plist_file(
-                wlmaker_arg_style_file_ptr));
-    } else {
-        style_dict_ptr = wlmcfg_dict_from_object(
-            wlmcfg_create_object_from_plist_data(
-                embedded_binary_style_default_data,
-                embedded_binary_style_default_size));
-    }
+    wlmcfg_dict_t *style_dict_ptr = wlmcfg_dict_from_object(
+        wlmaker_plist_load(
+            "style",
+            wlmaker_arg_style_file_ptr,
+            _wlmaker_style_fname_ptrs,
+            embedded_binary_style_data,
+            embedded_binary_style_size));
     if (NULL == style_dict_ptr) return EXIT_FAILURE;
-    BS_ASSERT(wlmcfg_decode_dict(
-                  style_dict_ptr,
-                  wlmaker_config_style_desc,
-                  &server_ptr->style));
+    if (!wlmcfg_decode_dict(
+            style_dict_ptr,
+            wlmaker_config_style_desc,
+            &server_ptr->style)) return EXIT_FAILURE;
     wlmcfg_dict_unref(style_dict_ptr);
 
+    server_ptr->root_menu_array_ptr = wlmcfg_array_from_object(
+        wlmaker_plist_load(
+            "root menu",
+            wlmaker_arg_root_menu_file_ptr,
+            _wlmaker_root_menu_fname_ptrs,
+            embedded_binary_root_menu_data,
+            embedded_binary_root_menu_size));
+    if (NULL == server_ptr->root_menu_array_ptr) return EXIT_FAILURE;
+    // TODO(kaeser@gubbe.ch): Uh, that's ugly...
+    server_ptr->root_menu_ptr = wlmaker_root_menu_create(
+        server_ptr,
+        &server_ptr->style.window,
+        &server_ptr->style.menu,
+        server_ptr->env_ptr);
+    if (NULL == server_ptr->root_menu_ptr) {
+        return EXIT_FAILURE;
+    }
+    wlmtk_menu_set_open(
+        wlmaker_root_menu_menu(server_ptr->root_menu_ptr),
+        false);
+
     wlmaker_action_handle_t *action_handle_ptr = wlmaker_action_bind_keys(
         server_ptr,
         wlmcfg_dict_get_dict(config_dict_ptr, wlmaker_action_config_dict_key));
@@ -384,7 +422,8 @@ int main(__UNUSED__ int argc, __UNUSED__ const char **argv)
             server_ptr, state_dict_ptr, &server_ptr->style);
         clip_ptr = wlmaker_clip_create(
             server_ptr, state_dict_ptr, &server_ptr->style);
-        task_list_ptr = wlmaker_task_list_create(server_ptr, &server_ptr->style);
+        task_list_ptr = wlmaker_task_list_create(
+            server_ptr, &server_ptr->style);
         if (NULL == dock_ptr || NULL == clip_ptr || NULL == task_list_ptr) {
             bs_log(BS_ERROR, "Failed to create dock, clip or task list.");
         } else {
@@ -400,6 +439,7 @@ int main(__UNUSED__ int argc, __UNUSED__ const char **argv)
     if (NULL != clip_ptr) wlmaker_clip_destroy(clip_ptr);
     if (NULL != dock_ptr) wlmaker_dock_destroy(dock_ptr);
     wlmaker_action_unbind_keys(action_handle_ptr);
+    wlmcfg_array_unref(server_ptr->root_menu_array_ptr);
     wlmaker_server_destroy(server_ptr);
 
     bs_subprocess_t *sp_ptr;
