@@ -20,6 +20,7 @@
 
 #include "xdg_shell.h"
 
+#include "tl_menu.h"
 #include "xdg_popup.h"
 
 #include <wlr/version.h>
@@ -40,6 +41,9 @@ typedef struct {
     /** The corresponding wlroots XDG toplevel. */
     struct wlr_xdg_toplevel   *wlr_xdg_toplevel_ptr;
 
+    /** The toplevel's window menu. */
+    wlmaker_tl_menu_t         *tl_menu_ptr;
+
     /** Listener for the `destroy` signal of the `wlr_xdg_toplevel::events`. */
     struct wl_listener        destroy_listener;
     /** Listener for the `new_popup` signal of the `wlr_xdg_surface`. */
@@ -161,16 +165,23 @@ wlmtk_window_t *wlmtk_window_create_from_xdg_toplevel(
     wlmtk_window_t *wlmtk_window_ptr = wlmtk_window_create(
         &surface_ptr->super_content,
         &server_ptr->style.window,
+        &server_ptr->style.menu,
         server_ptr->env_ptr);
     if (NULL == wlmtk_window_ptr) {
         xdg_toplevel_surface_destroy(surface_ptr);
         return NULL;
     }
-    wl_signal_emit(&server_ptr->window_created_event, wlmtk_window_ptr);
 
+    surface_ptr->tl_menu_ptr = wlmaker_tl_menu_create(
+        wlmtk_window_ptr, server_ptr);
+    if (NULL == surface_ptr->tl_menu_ptr) {
+        xdg_toplevel_surface_destroy(surface_ptr);
+        return NULL;
+    }
+
+    wl_signal_emit(&server_ptr->window_created_event, wlmtk_window_ptr);
     bs_log(BS_INFO, "Created window %p for wlmtk XDG toplevel surface %p",
            wlmtk_window_ptr, surface_ptr);
-
     return wlmtk_window_ptr;
 }
 
@@ -206,7 +217,7 @@ xdg_toplevel_surface_t *xdg_toplevel_surface_create(
 
     if (!wlmtk_content_init(
             &xdg_tl_surface_ptr->super_content,
-            xdg_tl_surface_ptr->surface_ptr,
+            wlmtk_surface_element(xdg_tl_surface_ptr->surface_ptr),
             server_ptr->env_ptr)) {
         xdg_toplevel_surface_destroy(xdg_tl_surface_ptr);
         return NULL;
@@ -421,6 +432,12 @@ void handle_destroy(struct wl_listener *listener_ptr,
     wl_signal_emit(
         &xdg_tl_surface_ptr->server_ptr->window_destroyed_event,
         xdg_tl_surface_ptr->super_content.window_ptr);
+
+    if (NULL != xdg_tl_surface_ptr->tl_menu_ptr) {
+        wlmaker_tl_menu_destroy(xdg_tl_surface_ptr->tl_menu_ptr);
+        xdg_tl_surface_ptr->tl_menu_ptr = NULL;
+    }
+
     wlmtk_window_destroy(xdg_tl_surface_ptr->super_content.window_ptr);
     xdg_toplevel_surface_destroy(xdg_tl_surface_ptr);
 }
@@ -478,12 +495,12 @@ void handle_surface_map(
     xdg_toplevel_surface_t *xdg_tl_surface_ptr = BS_CONTAINER_OF(
         listener_ptr, xdg_toplevel_surface_t, surface_map_listener);
 
-    wlmtk_workspace_t *wlmtk_workspace_ptr =
+    wlmtk_workspace_t *workspace_ptr =
         wlmtk_root_get_current_workspace(
             xdg_tl_surface_ptr->server_ptr->root_ptr);
 
     wlmtk_workspace_map_window(
-        wlmtk_workspace_ptr,
+        workspace_ptr,
         xdg_tl_surface_ptr->super_content.window_ptr);
 }
 
@@ -665,7 +682,7 @@ void handle_toplevel_request_resize(
  * Handler for the `request_show_window_menu` signal.
  *
  * @param listener_ptr
- * @param data_ptr
+ * @param data_ptr            struct wlr_xdg_toplevel_show_window_menu_event.
  */
 void handle_toplevel_request_show_window_menu(
     struct wl_listener *listener_ptr,
@@ -676,9 +693,10 @@ void handle_toplevel_request_show_window_menu(
         xdg_toplevel_surface_t,
         toplevel_request_show_window_menu_listener);
 
-    // TODO(kaeser@gubbe.ch): Implement.
-    bs_log(BS_WARNING, "Unimplemented: request_show_window_menu for XDG "
-           "toplevel %p", xdg_tl_surface_ptr);
+    wlmtk_window_menu_set_enabled(
+        xdg_tl_surface_ptr->super_content.window_ptr,
+        true);
+
 }
 
 /* ------------------------------------------------------------------------- */
