@@ -30,13 +30,13 @@ extern "C" {
 /**
  * Creates a toolkit window with the XDG surface as content.
  *
- * @param wlr_xdg_surface_ptr
+ * @param wlr_xdg_toplevel_ptr
  * @param server_ptr
  *
  * @return The window, or NULL on error.
  */
 wlmtk_window_t *wlmtk_window_create_from_xdg_toplevel(
-    struct wlr_xdg_surface *wlr_xdg_surface_ptr,
+    struct wlr_xdg_toplevel *wlr_xdg_toplevel_ptr,
     wlmaker_server_t *server_ptr);
 
 #ifdef __cplusplus
